from lex.fluent.slot import Slot
from lex.fluent.intent import Intent, IntentSlot
from lex import LexBotManager, LexIntentManager, LexSlotManager, LexPlayer


s = Slot(Name='testslot').with_value('AMAZON.NUMBER').apply()
sm = LexSlotManager()
slot_j = sm.get_slot_type(Name='testslot')
slot = Slot.from_json(slot_j)
slot.with_description('test description').apply()
slot_j = sm.get_slot_type(Name='testslot')
slot = Slot.from_json(slot_j)
print(slot.to_json())
# s = Slot(Name='test_timeslot').with_description('test_descript').apply()



class WithMessage:
    def __init__(self, **kwargs):
        self.group_number = kwargs.get('GroupNumber')
        self.max_attempts = kwargs.get('MaxAttempts')
        self.messages = kwargs.get('Messages', [])

    def _to_message(self, content_type, content, group_number):
        message_j = {'contentType': content_type,
                     'content': content}
        if group_number:
            message_j['groupNumber'] = group_number
        return message_j

    def to_dict(self):
        data = {}
        data['messages'] = self.messages
        if self.max_attempts:
            data['maxAttempts'] = self.max_attempts
        if self.response_card:
            data['responseCard'] = self.response_card
        return data

    def with_max_attempts(self, max_attempts):
        self.max_attempts = max_attempts
        return self

    def with_response_card(self, response_card):
        self.response_card = response_card
        return self

    def with_message(self, content_type, content, group_number=None):
        self.messages.append(self._to_message(content_type, content, group_number))
        return self

    @staticmethod
    def from_json(message_j):
        w = WithMessage()
        if 'maxAttempts' in message_j.keys():
            w.with_max_attempts(message_j['maxAttempts'])
        if 'responseCard' in message_j.keys():
            w.with_response_card(message_j['responseCard'])
        if 'messages' in message_j.keys():
            for msg in message_j['messages']:
                w.with_message(msg['contentType'], msg['content'], msg['groupNumber'])
        return w



import json


class DictUtils:
    @staticmethod
    def are_same(dict1, dict2):
        d1 = json.dumps(dict1, sort_keys=True, indent=2)
        d2 = json.dumps(dict2, sort_keys=True, indent=2)
        return d1 == d2



class BaseBot(object):
    def __init__(self):
        if not hasattr(self, 'bot_name'):
            self.bot_name = 'BaseBot'

    def on_fulfilled(self):
        pass

    def on_failed(self):
        pass

    def on_transition_in(self):
        pass

    def on_transition_out(self):
        pass

    def on_cancel(self):
        pass

    def on_misunderstood(self):
        pass

    def register(self):
        print("{}: Bot Registered Successfully".format(self.bot_name))



from lex import LexSlotManager
import json
from collections import namedtuple

import json

class Generic:
    @classmethod
    def from_dict(cls, dict):
        obj = cls()
        obj.__dict__.update(dict)
        return obj

class EnumerationValue:
    def __init__(self, **kwargs):
        self.value = kwargs.get('Value')
        self.synonyms = kwargs.get('Synonyms')


class Slot:
    def __init__(self, **kwargs):
        self.name = kwargs.get('Name')
        self.description = kwargs.get('Description')
        self.checksum = kwargs.get('Checksum')
        self.value = kwargs.get('Value')
        self.version = kwargs.get('Version', '$LATEST')
        self.create_version = kwargs.get('CreateVersion', False)
        self.value_selection_strategy = kwargs.get('ValueSelectionStrategy', 'ORIGINAL_VALUE')
        self.enumeration_values = kwargs.get('EnumerationValues', [])
        self.slot_manager = LexSlotManager()

    def with_name(self, name):
        self.name = name
        return self

    def with_description(self, description):
        self.description = description
        return self

    def with_checksum(self, checksum):
        self.checksum = checksum
        return self

    def with_version(self, version):
        self.version = version
        return self

    def with_value(self, value):
        self.value = value
        return self

    def with_create_version(self, create_version):
        self.create_version = create_version
        return self

    def with_value_selection_strategy(self, strategy):
        self.value_selection_strategy = strategy
        return self

    def with_enumeration_value(self, enumeration_value):
        self.enumeration_values.append({'value': enumeration_value.value,
                                        'synonyms': enumeration_value.synonyms})
        return self

    def to_json(self):
        slot_j = {"name": self.name}
        if self.create_version:
            slot_j['createVersion'] = self.create_version
        if self.value_selection_strategy:
            slot_j['valueSelectionStrategy'] = self.value_selection_strategy
        if self.description:
            slot_j['description'] = self.description
        if self.enumeration_values:
            slot_j['enumerationValues'] = self.enumeration_values
        if self.checksum:
            slot_j['checksum'] = self.checksum
        return slot_j

    @staticmethod
    def from_json(slot_j):
        s = Slot()
        if 'name' in slot_j.keys():
            s.with_name(slot_j['name'])
        if 'description' in slot_j.keys():
            s.with_description(slot_j['description'])
        if 'checksum' in slot_j.keys():
            s.with_checksum(slot_j['checksum'])
        if 'version' in slot_j.keys():
            s.with_checksum(slot_j['version'])
        if 'createVersion' in slot_j.keys():
            s.with_create_version(slot_j['createVersion'])
        if 'valueSelectionStrategy' in slot_j.keys():
            s.with_value_selection_strategy(slot_j['valueSelectionStrategy'])
        if 'enumerationValues' in slot_j.keys():
            for v in slot_j['enumerationValues']:
                s.enumeration_values.append(EnumerationValue(Value=v['value'],
                                                             Synonyms=v['synonyms']))
        return s

    def apply(self):
        self.slot_manager.upsert(self.to_json())
        return self

    def get(self):
        slot_j = self.slot_manager.get_slot_type(Name=self.name, Version=self.version)
        return Slot.from_json(slot_j)



import boto3
from lex.message import WithMessage


class AbortStatement(WithMessage):
    def __init__(self, **kwargs):
        super(AbortStatement, self).__init__(**kwargs)
        self.key = 'abortStatement'
        self.max_attempts = None


class ClarificationPrompt(WithMessage):
    def __init__(self, **kwargs):
        super(ClarificationPrompt, self).__init__(**kwargs)
        self.key = 'clarificationPrompt'


class Bot:
    def __init__(self, **kwargs):
        self.name = kwargs.get('Name')
        self.description = kwargs.get('Description')
        self.intents = kwargs.get('Intents', [])
        self.clarification_prompt = kwargs.get('ClarificationPrompt')
        self.abort_statement = kwargs.get('AbortStatement')
        self.idle_session_ttl_in_seconds = kwargs.get('IdleSessionTtlInSeconds')
        self.voice_id = kwargs.get('VoiceId')
        self.checksum = kwargs.get('Checksum')
        self.processBehavior = kwargs.get('ProcessBehavior', 'SAVE')
        self.locale = kwargs.get('Locale', 'en-US')
        self.childDirected = kwargs.get('ChildDirected', False)
        self.createVersion = kwargs.get('CreateVersion', False)

    def with_name(self, name):
        self.name = name
        return self

    def with_description(self, description):
        self.description = description
        return self

    def with_checksum(self, checksum):
        self.checksum = checksum
        return self

    def with_value(self, value):
        self.value = value
        return self

    def with_synonyms(self, synonyms):
        self.synonyms = synonyms
        return self

    def with_create_version(self, create_version):
        self.create_version = create_version
        return self

    def with_value_selection_strategy(self, strategy):
        self.value_selection_strategy = strategy
        return self

    def

    def apply(self):
        client = boto3.client('lex-models')
        slot_j = { "name": self.name,
                   "enumerationValues": [ { "value": self.value } ],
                   "valueSelectionStrategy": self.value_selection_strategy}
        if self.description:
            slot_j['description'] = self.description
        if self.synonyms:
            slot_j['enumerationValues']['synonyms'] = self.synonyms
        if self.checksum:
            slot_j['checksum'] = self.checksum
        client.put_slot_type(**slot_j)



from lex.fluent.message import WithMessage
from lex.fluent.slot import Slot
from lex import LexIntentManager


class DialogCodeHook:
    def __init__(self, **kwargs):
        self.data = {}
        self.data['uri'] = kwargs.get('Uri')
        self.data['messageVersion'] = kwargs.get('MessageVersion')

    def to_dict(self):
        return self.data


class FulfillmentActivity:
    def __init__(self, **kwargs):
        self.type = kwargs.get('Type')
        self.code_hook = kwargs.get('CodeHook')

    def with_code_hook(self, code_hook):
        self.code_hook = code_hook
        return self

    def with_type(self, type):
        self.type = type
        return self

    def to_dict(self):
        data = {}
        data['codeHook'] = self.code_hook.to_dict()
        data['type'] = self.type
        return data


class ConclusionStatement(WithMessage):
    def __init__(self, **kwargs):
        super(ConclusionStatement, self).__init__(**kwargs)
        self.key = 'conclusionStatement'


class ConfirmationPrompt(WithMessage):
    def __init__(self, **kwargs):
        super(ConfirmationPrompt, self).__init__(**kwargs)
        self.key = 'confirmationPrompt'


class RejectionStatement(WithMessage):
    def __init__(self, **kwargs):
        super(RejectionStatement, self).__init__(**kwargs)
        self.key = 'rejectionStatement'
        self.max_attempts = None


class Prompt(WithMessage):
    def __init__(self, **kwargs):
        super(Prompt, self).__init__(**kwargs)
        self.key = 'prompt'


class FollowUpPrompt:
    def __init__(self, **kwargs):
        self.prompt = kwargs.get('Prompt')
        self.rejection_statement = kwargs.get('RejectionStatement')

    def with_prompt(self, prompt):
        self.prompt = prompt
        return self

    def with_rejection_statement(self, rejection_statement):
        self.rejection_statement = rejection_statement
        return self

    def to_dict(self):
        data = {}
        if self.prompt:
            data['prompt'] = self.prompt.to_dict()
        if self.rejection_statement:
            data['rejection_statement'] = self.rejection_statement.to_dict()
        return data


class IntentSlot(Slot):
    def __new__(cls, **kwargs):
        if kwargs.get('Slot'):
            s = kwargs.get('Slot')
            s.__class__ = IntentSlot
            return s

    def __init__(self, **kwargs):
        if 'name' not in self.__dict__.keys():
            self.name = kwargs.get('Name')
        if 'description' not in self.__dict__.keys():
            self.description = kwargs.get('Description')
        if 'slot_constraint' not in self.__dict__.keys():
            self.slot_constraint = kwargs.get('SlotConstraint')
        if 'slot_type' not in self.__dict__.keys():
            self.slot_type = kwargs.get('SlotType')
        if 'create_version' in self.__dict__:
            del self.__dict__['create_version']
        self.slot_type_version = kwargs.get('SlotTypeVersion')
        self.value_elicitation_prompt = {"messages": []}
        self.max_attempts = kwargs.get('PromptMaxAttempts', 3)
        self.priority = kwargs.get('Priority')
        self.sample_utterances = kwargs.get('SampleUtterances')
        self.response_card = kwargs.get('ResponseCard')

    def with_name(self, name):
        self.name = name
        return self

    def with_description(self, description):
        self.description = description
        return self

    def with_slot_constraint(self, slot_constraint):
        self.slot_constraint = slot_constraint
        return self

    def with_slot_type(self, slot_type):
        self.slot_type = slot_type
        return self

    def with_slot_type_version(self, slot_type_version):
        self.slot_type_version = slot_type_version
        return self

    def with_value_elicitation_prompt(self, value_elicitation_prompt):
        self.value_elicitation_prompt = value_elicitation_prompt
        return self

    def with_priority(self, priority):
        self.priority = priority
        return self

    def with_sample_utterances(self, sample_utterances):
        self.sample_utterances = sample_utterances
        return self

    def with_response_card(self, response_card):
        self.response_card = response_card
        return self

    def with_value_elicitation_prompt_message(self, content_type, content, group_number):
        self.value_elicitation_prompt['messages'].append(
           self._to_message(content_type, content, group_number))
        return self

    def with_prompt_max_attempts(self, max_attempts):
        self.value_elicitation_prompt['maxAttempts'] = max_attempts
        return self

    def with_prompt_response_card(self, response_card):
        self.value_elicitation_prompt['responseCard'] = response_card
        return self

    def to_dict(self):
        data = {}
        data['name'] = self.name
        data['description'] = self.description
        data['slotConstraint'] = self.slot_constraint
        data['slotType'] = self.slot_type
        return data


class Intent:
    def __init__(self, **kwargs):
        self.name = kwargs.get('Name')
        self.description = kwargs.get('Description')
        self.checksum = kwargs.get('Checksum')
        self.value = kwargs.get('Value')
        self.create_version = kwargs.get('CreateVersion', False)
        self.value_selection_strategy = kwargs.get('ValueSelectionStrategy', 'ORIGINAL_VALUE')
        self.synonyms = kwargs.get('Synonyms')
        self.confirmation_prompt = kwargs.get('ConfirmationPrompt', {})
        self.rejection_statement = kwargs.get('RejectionStatement', {})
        self.conclusion_statement = kwargs.get('ConclusionStatement', {})
        self.follow_up_prompt = kwargs.get('FollowUpPrompt', {})
        self.parentIntentSignature = kwargs.get('ParentIntentSignature')
        self.checksum = kwargs.get('Checksum')
        self.slots = kwargs.get('Slots', [])
        self.dialog_hook = kwargs.get('DialogHook')
        self.fulfillment_activity = kwargs.get('FulfillmentActivity', {})
        self.sample_utterances = kwargs.get('SampleUtterances', [])
        self.intent_manager = LexIntentManager()

    def with_name(self, name):
        self.name = name
        return self

    def with_description(self, description):
        self.description = description
        return self

    def with_checksum(self, checksum):
        self.checksum = checksum
        return self

    def with_value(self, value):
        self.value = value
        return self

    def with_synonyms(self, synonyms):
        self.synonyms = synonyms
        return self

    def with_create_version(self, create_version):
        self.create_version = create_version
        return self

    def with_value_selection_strategy(self, strategy):
        self.value_selection_strategy = strategy
        return self

    def with_confirmation_prompt(self, confirmation_prompt):
        self.confirmation_prompt = confirmation_prompt
        return self

    def with_rejection_statement(self, rejection_statement):
        self.rejection_statement = rejection_statement
        return self

    def with_follow_up_prompt(self, follow_up_prompt):
        self.follow_up_prompt = follow_up_prompt
        return self

    def with_conclusion_statement(self, conclusion_statement):
        self.conclusion_statement = conclusion_statement
        return self

    def with_dialog_hook(self, dialog_hook):
        self.dialog_hook = dialog_hook
        return self

    def with_intent_slot(self, slot):
        self.slots.append(slot)
        return self

    def with_slot(self, slot):
        self.slots.append(IntentSlot(Slot=slot).to_dict())
        return self

    def with_sample_utterances(self, sample_utterance):
        self.sample_utterances.append(sample_utterance)
        return self

    def with_fulfillment_activity(self, fulfillment_activity):
        self.fulfillment_activity = fulfillment_activity
        return self

    def to_json(self):
        intent_j = {"name": self.name}
        if self.description:
            intent_j['description'] = self.description
        if self.sample_utterances:
            intent_j['sampleUtterances'] = self.sample_utterances
        if self.slots:
            intent_j['slots'] = self.slots
        if self.rejection_statement:
            intent_j['rejectionStatement'] = self.rejection_statement.to_dict()
        if self.confirmation_prompt:
            intent_j['confirmationPrompt'] = self.confirmation_prompt.to_dict()
        if self.conclusion_statement:
            intent_j['conclusionStatement'] = self.conclusion_statement.to_dict()
        if self.fulfillment_activity:
            intent_j['fulfillmentActivity'] = self.fulfillment_activity.to_dict()
        if self.follow_up_prompt:
            intent_j['followUpPrompt'] = self.follow_up_prompt.to_dict()
        if self.dialog_hook:
            intent_j['dialogCodeHook'] = self.dialog_hook.to_dict()
        return intent_j

    def from_json(intent_j):
        intent = Intent()
        if 'name' in intent_j.keys():
            intent.with_name(intent_j['name'])
        if 'checksum' in intent_j.keys():
            intent.with_checksum(intent_j['checksum'])
        if 'description' in intent_j.keys():
            intent.with_description(intent_j['description'])
        if 'sampleUtterances' in intent_j.keys():
            for u in intent_j['sampleUtterances']:
                intent.with_sample_utterance(u)
        if 'slots' in intent_j.keys():
            for s in intent_j['slots']:
                intent.with_intent_slot(s)
        if 'dialogCodeHook' in intent_j.keys():
            d = intent_j['dialogCodeHook']
            intent.with_code_hook(DialogCodeHook(Uri=d['uri'],
                                                 MessageVersion=d['messageVersion']))
        if 'rejectionStatement' in intent_j.keys():
            r = WithMessage.from_json(intent_j['rejectionStatment'])
            intent.with_rejection_statement(r)

        if 'conclusionStatement' in intent_j.keys():
            c = WithMessage.from_json(intent_j['conclusionStatement'])
            intent.with_confirmation_prompt(c)

        if 'confirmationPrompt' in intent_j.keys():
            c = WithMessage.from_json(intent_j['confirmationPrompt'])
            intent.with_confirmation_prompt(c)

        if 'followUpPrompt' in intent_j.keys():
            f = FollowUpPrompt()
            if 'rejectionStatement' in intent_j['followUpPrompt']:
                r = WithMessage.from_json(intent_j['followUpPrompt']['rejectionStatment'])
                f.with_rejection_statment(r)
            if 'prompt' in intent_j['followUpPrompt']:
                p = WithMessage.from_json(intent_j['followUpPrompt']['prompt'])
                f.with_prompt(p)
            intent.with_follow_up_prompt(f)

        if 'fulfillmentActivity' in intent_j.keys():
            f = FulfillmentActivity()
            d = intent_j['fulfillmentActivity']['dialogCodeHook']
            f.with_code_hook(DialogCodeHook(Uri=d['uri'],
                                            MessageVersion=d['messageVersion']))
            f.with_type(intent_j['fulfillmentActivity']['type'])
            intent.with_fulfillment_activity(f)
        return intent

    def apply(self):
        self.intent_manager.upsert(self.to_json())
        return self

    def get(self):
        intent_j = self.intent_manager.get_intent(Name=self.name, Version=self.version)
        return Intent.from_json(intent_j)
